//---------------------------------------------------------------
//---------------------------------------------------------------
//------------  Simple SGA				             ------------
//------------  - function optimization				 ------------
//------------  - binary representation              ------------  
//------------  - generational replacement strategy  ------------  
//---------------------------------------------------------------
//---------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <time.h>
#include <string.h>
#include <float.h>
//----------------
#include "general_sga.h"
#include "individual.h"
#include "generation_sga.h"

//------- Simple GA control parameters
int popSize;				// population size of the SGA
tind *population;			// population of the SGA
int maxEvaluations;			// stopping criterion - overall maximal number of fitness evaluations cummulated over all SGAs
int currEvaluations;		// current number of calculated fitness function evaluations
tstat statdata;				// population statistics of the SGAs
toverallstat overallStatData;
int rand_seed;				// seeds for pseudo-random generator
int maxGenes;				// fixed length of chromosomes



//----------------------------------------------------------------------
//------- overall initialisation
//------- allocates memory for individuals
void initAll(){
	int i;

	//------- Random number generator seed
	srand(rand_seed = 5);

	//------- SGA parameters
	nTournament = 4;		// parameter of tournament selection
	pCross = 50;			// probability of crossover [%]
	pMutation = 5;			// probability of mutation [%]

	//------- Initializes control parameters
	maxGenes = 80;
	popSize = 2024;	
	maxEvaluations = 400000;			
	currEvaluations = 0;
	overallStatData.favg = -1.0e100;
	overallStatData.fbest = -1.0e100;
	overallStatData.popSize = 0;
	overallStatData.when = 0;
	population = NULL;
}

//----------------------------------------------------------------------
//------- Final report
void finalReport(){
	int i;

	printf("\n\n======= FINAL REPORT =======");
	printf("\nTotal number of fitness evaluations: %d", currEvaluations);
	printf("\nBest-of-the-run solution: %.3f", overallStatData.fbest);
	printf("\nFound in population of size: %d", overallStatData.popSize);
	printf("\nNumber of FE needed to find it: %d", overallStatData.when);
	printf("\nThe best solution found: ");
	for(i=0; i<maxGenes; i++)
		printf("%d", population[statdata.indbest].genes[i]);
}

//--------------------------------------------------------
//-------------------- MAIN EVOLUTION --------------------
//--------------------------------------------------------
int main(int argc,char *argv[]){
	int i;
	int par1, par2;				// indexes of parents
	int individualsGenerated;   // a number of newly generated individuals

	//------- Initialization
	initAll();

	//------- initialization of the first population
	if(initPopulation(&currEvaluations) == 0)
		return 1;
	if(overallStatData.fbest < statdata.fbest){
		overallStatData.fbest = statdata.fbest;
		overallStatData.favg = statdata.favg;
		overallStatData.popSize = popSize;
		overallStatData.when = currEvaluations;
	}

	//-----------------------------------------------
	//------- Main loop of the SGA
	//-----------------------------------------------
	do{//------- Perform one generation with the population
		doGeneration(&currEvaluations);
		if(overallStatData.fbest < statdata.fbest){
			overallStatData.fbest = statdata.fbest;
			overallStatData.favg = statdata.favg;
			overallStatData.popSize = popSize;
			overallStatData.when = currEvaluations;
		}
	}while(currEvaluations < maxEvaluations);

	//------- Final report
	finalReport();

	//------- Cleaning
	destroyPopulation();

	//-------
	printf("\n\nFinished ...");
	getch();
	return 0;
}
